"""CLI for autonomous-claude."""

import json
import shutil
import subprocess
from pathlib import Path

import typer
from rich.console import Console
from rich.markdown import Markdown
from rich.panel import Panel

from . import __version__
from .agent import run_agent_loop
from .client import generate_app_spec, generate_task_spec, ensure_claude_cli
from .config import get_config, SPEC_FILE, AUTONOMOUS_CLAUDE_DIR

console = Console()

# Legacy path for migration
LEGACY_FEATURES_FILE = f"{AUTONOMOUS_CLAUDE_DIR}/features.json"


def check_gh_auth() -> bool:
    """Check if GitHub CLI is installed and authenticated."""
    if not shutil.which("gh"):
        return False
    result = subprocess.run(["gh", "auth", "status"], capture_output=True, text=True)
    return result.returncode == 0


def get_repo_remote(project_dir: Path) -> str | None:
    """Get the GitHub repo from git remote origin."""
    result = subprocess.run(
        ["git", "remote", "get-url", "origin"],
        capture_output=True, text=True, cwd=project_dir
    )
    if result.returncode != 0:
        return None
    # Extract owner/repo from URL
    url = result.stdout.strip()
    # Handle both HTTPS and SSH URLs
    if "github.com" in url:
        # https://github.com/owner/repo.git or git@github.com:owner/repo.git
        parts = url.replace(":", "/").replace(".git", "").split("/")
        if len(parts) >= 2:
            return f"{parts[-2]}/{parts[-1]}"
    return None


def ensure_github_repo(project_dir: Path) -> str | None:
    """Ensure project has a GitHub repo. Returns repo name or None on failure."""
    # Check gh auth
    if not check_gh_auth():
        console.print("[red]GitHub CLI not authenticated.[/red]")
        console.print("[dim]Run: gh auth login[/dim]")
        return None

    # Check if already has remote
    existing = get_repo_remote(project_dir)
    if existing:
        console.print(f"[dim]Using repo:[/dim] {existing}")
        return existing

    # Need to create repo - check if git is initialized
    git_dir = project_dir / ".git"
    if not git_dir.exists():
        console.print("[dim]Initializing git...[/dim]")
        subprocess.run(["git", "init"], cwd=project_dir, capture_output=True)

    # Create private repo (without --push since directory may be empty)
    repo_name = project_dir.name
    console.print(f"[dim]Creating private repo:[/dim] {repo_name}")
    result = subprocess.run(
        ["gh", "repo", "create", repo_name, "--private", "--source=."],
        cwd=project_dir, capture_output=True, text=True
    )
    if result.returncode != 0:
        console.print(f"[red]Failed to create repo:[/red] {result.stderr}")
        return None

    return get_repo_remote(project_dir)


def migrate_features_to_issues(project_dir: Path) -> bool:
    """Migrate legacy features.json to GitHub Issues. Returns True if migrated."""
    features_file = project_dir / LEGACY_FEATURES_FILE
    if not features_file.exists():
        return False

    # Check if repo exists
    repo = get_repo_remote(project_dir)
    if not repo:
        console.print("[yellow]Cannot migrate: no GitHub remote configured.[/yellow]")
        console.print("[dim]Set up a remote first: gh repo create[/dim]")
        return False

    console.print("[cyan]Migrating features.json to GitHub Issues...[/cyan]")

    try:
        features = json.loads(features_file.read_text())
    except json.JSONDecodeError as e:
        console.print(f"[red]Failed to parse features.json: {e}[/red]")
        return False

    # Ensure label exists
    subprocess.run(
        ["gh", "label", "create", "autonomous-claude",
         "--description", "Managed by autonomous-claude", "--force"],
        cwd=project_dir, capture_output=True
    )

    migrated = 0
    for f in features:
        if not f.get("passes"):
            description = f.get("description", "Untitled feature")
            steps = f.get("steps", [])
            body = "## Steps to verify\n" + "\n".join(f"- [ ] {s}" for s in steps) if steps else ""

            result = subprocess.run(
                ["gh", "issue", "create",
                 "--title", f"Feature: {description}",
                 "--body", body,
                 "--label", "autonomous-claude"],
                cwd=project_dir, capture_output=True, text=True
            )
            if result.returncode == 0:
                migrated += 1
                console.print(f"[dim]  Created: {result.stdout.strip()}[/dim]")

    # Backup old file
    features_file.rename(features_file.with_suffix(".json.migrated"))
    console.print(f"[green]Migrated {migrated} pending feature(s) to GitHub Issues[/green]")
    return True


def confirm_spec(spec: str, title: str = "Spec", project_dir: Path | None = None, auto_accept: bool = False) -> str:
    """Display spec for user confirmation or modification."""
    console.print()
    console.print(Panel(Markdown(spec), title=title, border_style="dim", padding=(1, 2)))

    if auto_accept:
        console.print("[dim]Auto-accepting spec (--yes)[/dim]")
        return spec

    while True:
        choice = typer.prompt("Accept?", default="y").lower().strip()
        if choice in ("y", "yes", ""):
            return spec
        feedback = choice if len(choice) > 1 else typer.prompt("What needs changing?")
        console.print("[dim]Updating spec...[/dim]")
        spec = generate_app_spec(f"{spec}\n\n## Changes Requested\n{feedback}", project_dir=project_dir)
        console.print()
        console.print(Panel(Markdown(spec), title=title, border_style="dim", padding=(1, 2)))


app = typer.Typer(
    name="autonomous-claude",
    help="Build apps autonomously with Claude Code CLI.",
    add_completion=False,
    no_args_is_help=False,
)


def version_callback(value: bool):
    if value:
        print(f"autonomous-claude {__version__}")
        raise typer.Exit()


def has_open_issues(project_dir: Path) -> tuple[bool, int]:
    """Check if project has open autonomous-claude issues. Returns (has_issues, count)."""
    result = subprocess.run(
        ["gh", "issue", "list", "--label", "autonomous-claude", "--state", "open", "--json", "number"],
        capture_output=True, text=True, cwd=project_dir
    )
    if result.returncode != 0:
        return False, 0
    try:
        issues = json.loads(result.stdout)
        return len(issues) > 0, len(issues)
    except json.JSONDecodeError:
        return False, 0


def run_default(
    instructions: str | None,
    model: str | None,
    max_sessions: int | None,
    timeout: int | None,
    sandbox: bool = True,
    auto_accept: bool = False,
):
    """Start new project or add features to existing one."""
    if not sandbox:
        ensure_claude_cli()

    project_dir = Path.cwd()
    config = get_config()

    # Check for legacy features.json and migrate if needed
    legacy_file = project_dir / LEGACY_FEATURES_FILE
    if legacy_file.exists():
        migrate_features_to_issues(project_dir)

    # Check GitHub auth
    if not check_gh_auth():
        if not shutil.which("gh"):
            console.print("[red]GitHub CLI (gh) not installed.[/red]")
            console.print("[dim]Install: https://cli.github.com/[/dim]")
        else:
            console.print("[red]GitHub CLI not authenticated.[/red]")
            console.print("[dim]Run: gh auth login[/dim]")
        raise typer.Exit(1)

    # Check if project has open issues (enhancement mode)
    has_issues, issue_count = has_open_issues(project_dir)

    if has_issues:
        # Enhancement mode
        console.print(f"[yellow]Warning:[/yellow] {issue_count} open issue(s).")
        console.print("[dim]Use '--continue' to continue without adding new features.[/dim]")
        if not auto_accept and not typer.confirm("Proceed with adding new features?", default=False):
            raise typer.Exit(0)

        if not instructions:
            if auto_accept:
                console.print("[red]--yes requires instructions argument[/red]")
                raise typer.Exit(1)
            instructions = typer.prompt("What do you want to add")

        console.print(f"[dim]Adding to:[/dim] {project_dir}")
        console.print("[dim]Generating task spec...[/dim]")
        spec = generate_task_spec(instructions, project_dir=project_dir)
        spec = confirm_spec(spec, title="Task Spec", project_dir=project_dir, auto_accept=auto_accept)

        run_agent_loop(
            project_dir=project_dir.resolve(),
            model=model,
            max_sessions=max_sessions or config.max_sessions,
            app_spec=spec,
            timeout=timeout or config.timeout,
            is_enhancement=True,
            sandbox=sandbox,
            interactive=not auto_accept,
        )
    else:
        # New project mode
        if not instructions:
            if auto_accept:
                console.print("[red]--yes requires instructions argument[/red]")
                raise typer.Exit(1)
            instructions = typer.prompt("Describe what you want to build")

        # Check if instructions is a file path
        try:
            spec_path = Path(instructions)
            is_file = spec_path.exists() and spec_path.is_file()
        except OSError:
            is_file = False

        if is_file:
            console.print(f"[dim]Reading spec from:[/dim] {spec_path}")
            spec = spec_path.read_text()
        else:
            console.print("[dim]Generating spec...[/dim]")
            spec = generate_app_spec(instructions, project_dir=project_dir)

        spec = confirm_spec(spec, title="App Spec", project_dir=project_dir, auto_accept=auto_accept)

        # Ensure GitHub repo exists for new projects
        repo = ensure_github_repo(project_dir)
        if not repo:
            console.print("[red]Cannot proceed without GitHub repo.[/red]")
            raise typer.Exit(1)

        run_agent_loop(
            project_dir=project_dir.resolve(),
            model=model,
            max_sessions=max_sessions or config.max_sessions,
            app_spec=spec,
            timeout=timeout or config.timeout,
            sandbox=sandbox,
            interactive=not auto_accept,
        )


@app.callback(invoke_without_command=True)
def main(
    ctx: typer.Context,
    instructions: str | None = typer.Argument(None, help="What to build or add"),
    continue_project: bool = typer.Option(False, "--continue", "-c", help="Continue existing features"),
    yes: bool = typer.Option(False, "--yes", "-y", help="Auto-accept spec (non-interactive mode)"),
    no_sandbox: bool = typer.Option(False, "--no-sandbox", help="Run without Docker sandbox"),
    model: str | None = typer.Option(None, "--model", "-m", help="Claude model"),
    max_sessions: int | None = typer.Option(None, "--max-sessions", "-n", help="Max sessions"),
    timeout: int | None = typer.Option(None, "--timeout", "-t", help="Session timeout (seconds)"),
    version: bool = typer.Option(False, "--version", "-v", callback=version_callback, is_eager=True, help="Show version"),
):
    """Build apps autonomously with Claude Code CLI."""
    if ctx.invoked_subcommand:
        return

    # Handle "update" passed as positional arg
    if instructions == "update":
        update()
        return

    sandbox = get_config().sandbox_enabled and not no_sandbox

    if continue_project:
        run_continue(model, max_sessions, timeout, sandbox, auto_accept=yes)
    else:
        run_default(instructions, model, max_sessions, timeout, sandbox, auto_accept=yes)


def run_continue(model: str | None, max_sessions: int | None, timeout: int | None, sandbox: bool = True, auto_accept: bool = False):
    """Continue work on existing features."""
    if not sandbox:
        ensure_claude_cli()

    project_dir = Path.cwd()

    # Check for legacy features.json and migrate if needed
    legacy_file = project_dir / LEGACY_FEATURES_FILE
    if legacy_file.exists():
        migrate_features_to_issues(project_dir)

    # Check GitHub auth
    if not check_gh_auth():
        if not shutil.which("gh"):
            console.print("[red]GitHub CLI (gh) not installed.[/red]")
            console.print("[dim]Install: https://cli.github.com/[/dim]")
        else:
            console.print("[red]GitHub CLI not authenticated.[/red]")
            console.print("[dim]Run: gh auth login[/dim]")
        raise typer.Exit(1)

    # Check if there are open issues to work on
    has_issues, issue_count = has_open_issues(project_dir)
    if not has_issues:
        console.print("[yellow]No open issues to work on.[/yellow]")
        console.print("[dim]Run 'autonomous-claude \"description\"' to add new features.[/dim]")
        raise typer.Exit(0)

    console.print(f"[dim]Found {issue_count} open issue(s)[/dim]")

    spec = None
    if not (project_dir / SPEC_FILE).exists():
        console.print("[dim]No spec.md found.[/dim]")
        if auto_accept:
            console.print("[red]--yes with --continue requires existing spec.md[/red]")
            raise typer.Exit(1)
        description = typer.prompt("Briefly describe this project")
        console.print("[dim]Generating spec...[/dim]")
        spec = generate_app_spec(description, project_dir=project_dir)
        spec = confirm_spec(spec, title="App Spec", project_dir=project_dir, auto_accept=auto_accept)

    config = get_config()
    run_agent_loop(
        project_dir=project_dir.resolve(),
        model=model,
        max_sessions=max_sessions or config.max_sessions,
        app_spec=spec,
        timeout=timeout or config.timeout,
        sandbox=sandbox,
        interactive=not auto_accept,
    )


@app.command()
def update():
    """Update to latest version from PyPI."""
    import urllib.request

    console.print(f"Current: {__version__}")
    console.print("Checking for updates...")

    with urllib.request.urlopen("https://pypi.org/pypi/autonomous-claude/json", timeout=10) as r:
        latest = json.loads(r.read().decode())["info"]["version"]

    current_base = __version__.split(".dev")[0].split("+")[0]
    if current_base == latest:
        console.print(f"Up to date ({latest})")
        return

    console.print(f"Updating {__version__} → {latest}...")
    result = subprocess.run(["uv", "tool", "install", "--force", "autonomous-claude"], capture_output=True, text=True)
    if result.returncode == 0:
        console.print(f"Updated to {latest}")
    else:
        console.print(f"[red]Update failed: {result.stderr}[/red]")
        raise typer.Exit(1)


if __name__ == "__main__":
    app()



import json
import shutil
import subprocess
import threading
import time
from datetime import datetime
from importlib import resources
from pathlib import Path

from .client import ClaudeCLIClient
from .prompts import (
    get_initializer_prompt, get_coding_prompt,
    get_adoption_initializer_prompt, get_enhancement_initializer_prompt,
    copy_spec_to_project,
)
from . import ui
from .config import AUTONOMOUS_CLAUDE_DIR, LOGS_DIR, get_config

SKILLS_DIR = Path.home() / ".claude" / "skills"
RALPH_STATE_FILE = f"{AUTONOMOUS_CLAUDE_DIR}/.ralph-state"


def install_bundled_skills():
    SKILLS_DIR.mkdir(parents=True, exist_ok=True)
    with resources.as_file(resources.files("autonomous_claude") / "skills") as src:
        if not src.exists():
            return
        for skill in src.iterdir():
            if not skill.is_dir():
                continue
            dest = SKILLS_DIR / skill.name
            if dest.exists():
                continue
            shutil.copytree(skill, dest)
            if skill.name == "playwright-skill" and (dest / "package.json").exists():
                subprocess.run(["pnpm", "run", "setup"], cwd=dest, capture_output=True)


def setup_ralph_hook(project_dir: Path):
    config = get_config()
    if not config.ralph_enabled:
        return

    hooks_dir = project_dir / ".claude" / "hooks"
    hooks_dir.mkdir(parents=True, exist_ok=True)

    with resources.as_file(resources.files("autonomous_claude") / "hooks" / "stop_hook.sh") as src:
        if src.exists():
            dest = hooks_dir / "stop_hook.sh"
            shutil.copy(src, dest)
            dest.chmod(0o755)

    settings_file = project_dir / ".claude" / "settings.local.json"
    settings = {}
    if settings_file.exists():
        try:
            settings = json.loads(settings_file.read_text())
        except json.JSONDecodeError:
            pass

    settings["hooks"] = {
        "Stop": [{
            "matcher": "*",
            "hooks": [{
                "type": "command",
                "command": f"RALPH_MAX_ITERATIONS={config.ralph_max_iterations} .claude/hooks/stop_hook.sh"
            }]
        }]
    }
    settings_file.write_text(json.dumps(settings, indent=2))


def reset_ralph_state(project_dir: Path):
    state_file = project_dir / RALPH_STATE_FILE
    if state_file.exists():
        state_file.unlink()


def run_with_spinner(func, *args, label: str = "Running...", **kwargs):
    result, error = [None], [None]

    def run():
        try:
            result[0] = func(*args, **kwargs)
        except Exception as e:
            error[0] = e

    thread = threading.Thread(target=run)
    with ui.Spinner(label):
        thread.start()
        while thread.is_alive():
            thread.join(0.1)

    if error[0]:
        raise error[0]
    return result[0]


def get_open_issues(project_dir: Path) -> list[dict]:
    result = subprocess.run(
        ["gh", "issue", "list", "--label", "autonomous-claude", "--state", "open", "--json", "number,title,body"],
        capture_output=True, text=True, cwd=project_dir
    )
    if result.returncode != 0:
        return []
    try:
        return json.loads(result.stdout)
    except json.JSONDecodeError:
        return []


def all_issues_closed(project_dir: Path) -> bool:
    return len(get_open_issues(project_dir)) == 0


def has_existing_issues(project_dir: Path) -> bool:
    result = subprocess.run(
        ["gh", "issue", "list", "--label", "autonomous-claude", "--state", "all", "--json", "number", "--limit", "1"],
        capture_output=True, text=True, cwd=project_dir
    )
    if result.returncode != 0:
        return False
    try:
        return len(json.loads(result.stdout)) > 0
    except json.JSONDecodeError:
        return False


def write_session_log(path: Path, session_type: str, prompt: str, stdout: str, stderr: str, duration: float):
    with open(path, "w") as f:
        f.write(f"Type: {session_type}\nTime: {datetime.now().isoformat()}\nDuration: {duration:.1f}s\n")
        f.write(f"\n{'='*60}\nPROMPT:\n{'='*60}\n{prompt}\n")
        f.write(f"\n{'='*60}\nOUTPUT:\n{'='*60}\n{stdout or '(empty)'}\n")
        if stderr:
            f.write(f"\n{'='*60}\nSTDERR:\n{'='*60}\n{stderr}\n")


def run_session(project_dir: Path, model: str | None, prompt: str, timeout: int,
                session_type: str, spinner_label: str, sandbox: bool) -> float:
    logs_dir = project_dir / LOGS_DIR
    logs_dir.mkdir(parents=True, exist_ok=True)
    log_path = logs_dir / f"{datetime.now().strftime('%Y%m%d_%H%M%S')}_{session_type}.log"

    start = time.time()
    try:
        client = ClaudeCLIClient(project_dir=project_dir, model=model, timeout=timeout, sandbox=sandbox)
        stdout, stderr = run_with_spinner(client.query, prompt, label=spinner_label)
        duration = time.time() - start
        write_session_log(log_path, session_type, prompt, stdout, stderr, duration)
        ui.print_output(stdout, stderr)
    except subprocess.TimeoutExpired:
        duration = time.time() - start
        write_session_log(log_path, session_type, prompt, "", f"TIMEOUT after {timeout}s", duration)
        ui.print_timeout(timeout, duration)
    except Exception as e:
        duration = time.time() - start
        write_session_log(log_path, session_type, prompt, "", str(e), duration)
        ui.print_error(e, duration, session_type)

    return duration


def run_agent_loop(
    project_dir: Path,
    model: str | None = None,
    max_sessions: int | None = None,
    app_spec: str | None = None,
    timeout: int = 1800,
    is_adoption: bool = False,
    is_enhancement: bool = False,
    sandbox: bool = True,
    interactive: bool = True,
):
    project_dir.mkdir(parents=True, exist_ok=True)

    mcp_config = project_dir / ".mcp.json"
    if not mcp_config.exists():
        mcp_config.write_text('{"mcpServers": {}}\n')

    is_new_project = not has_existing_issues(project_dir)
    if is_new_project:
        run_with_spinner(install_bundled_skills, label="Installing skills...")

    setup_ralph_hook(project_dir)
    ui.print_header(project_dir, model)

    needs_initializer = is_new_project
    needs_enhancement_init = is_enhancement

    if is_new_project:
        if app_spec:
            copy_spec_to_project(project_dir, app_spec)
        ui.print_adoption_notice() if is_adoption else ui.print_new_project_notice()
    elif is_enhancement:
        if app_spec:
            copy_spec_to_project(project_dir, app_spec)
        ui.print_enhancement_notice()
    else:
        ui.print_resuming(project_dir)

    ui.print_separator()

    session_count = 0
    total_time = 0.0

    while True:
        if not needs_initializer and not needs_enhancement_init and all_issues_closed(project_dir):
            break

        session_count += 1
        if max_sessions and session_count > max_sessions:
            ui.print_max_sessions(max_sessions)
            break

        issues_before = len(get_open_issues(project_dir))

        if needs_enhancement_init:
            prompt, session_type, label = get_enhancement_initializer_prompt(), "enhancement_init", "Enhancement init..."
            needs_enhancement_init = False
        elif needs_initializer:
            if is_adoption:
                prompt, session_type, label = get_adoption_initializer_prompt(), "adoption_init", "Adoption init..."
            else:
                prompt, session_type, label = get_initializer_prompt(), "initializer", "Initializing..."
            needs_initializer = False
        else:
            prompt, session_type, label = get_coding_prompt(), "coding", "Coding..."

        reset_ralph_state(project_dir)

        print()
        duration = run_session(project_dir, model, prompt, timeout, session_type, label, sandbox)
        total_time += duration

        issues_after = len(get_open_issues(project_dir))
        issues_closed = issues_before - issues_after

        ui.print_issue_progress(project_dir, issues_closed, issues_after, duration, total_time)
        ui.print_separator()

        if all_issues_closed(project_dir):
            break

        if interactive and ui.wait_for_stop_signal():
            ui.print_user_stopped()
            return

    ui.print_complete(project_dir, session_count, total_time)



"""UI components for autonomous-claude."""

import platform
import select
import shutil
import subprocess
import sys
import termios
import time
import tty
from pathlib import Path

from rich.console import Console
from rich.markdown import Markdown
from rich.panel import Panel
from rich.progress import Progress, SpinnerColumn, TextColumn, TimeElapsedColumn
from rich.table import Table

from .config import get_config

console = Console()

LOGO = """[bold cyan]
╔═╗╦ ╦╔╦╗╔═╗╔╗╔╔═╗╔╦╗╔═╗╦ ╦╔═╗
╠═╣║ ║ ║ ║ ║║║║║ ║║║║║ ║║ ║╚═╗
╩ ╩╚═╝ ╩ ╚═╝╝╚╝╚═╝╩ ╩╚═╝╚═╝╚═╝
     [dim]Claude Code CLI[/dim][/bold cyan]
"""


def format_duration(s: float) -> str:
    if s < 60:
        return f"{s:.0f}s"
    m, s = int(s // 60), int(s % 60)
    if m < 60:
        return f"{m}m {s}s"
    return f"{m // 60}h {m % 60}m"


def play_notification() -> None:
    config = get_config()
    sound = Path(config.notification_sound)
    player = None

    if platform.system() == "Linux":
        player = shutil.which("paplay") or shutil.which("aplay")
    elif platform.system() == "Darwin":
        player = shutil.which("afplay")

    for i in range(config.notification_dings):
        if player and sound.exists():
            subprocess.Popen([player, str(sound)], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        else:
            sys.stdout.write("\a")
            sys.stdout.flush()
        if i < config.notification_dings - 1:
            time.sleep(config.notification_interval)




def print_header(project_dir: Path, model: str | None) -> None:
    console.print(LOGO)
    table = Table(show_header=False, box=None, padding=(0, 2))
    table.add_column(style="dim")
    table.add_column()
    table.add_row("Project", f"[bold]{project_dir}[/bold]")
    table.add_row("Model", model or "[dim]default[/dim]")
    console.print(table)
    console.print()


def print_new_project_notice() -> None:
    console.print("[yellow]Setting up new project...[/yellow]\n")


def print_adoption_notice() -> None:
    console.print("[yellow]Analyzing existing project...[/yellow]\n")


def print_enhancement_notice() -> None:
    console.print("[yellow]Adding features...[/yellow]\n")


def print_resuming(project_dir: Path) -> None:
    console.print("[green]Continuing...[/green]")


def print_issue_progress(project_dir: Path, issues_closed: int, open_count: int,
                         session_duration: float | None, total_time: float | None) -> None:
    """Print progress based on GitHub Issues."""
    console.print()

    if session_duration and total_time:
        console.print(f"[dim]Session: {format_duration(session_duration)} | Total: {format_duration(total_time)}[/dim]\n")

    if issues_closed > 0:
        console.print(f"[bold green]Closed {issues_closed} issue(s) this session[/bold green]")

    if open_count == 0:
        console.print("[bold green]All issues completed![/bold green]")
    else:
        style = "yellow" if open_count <= 3 else "white"
        console.print(f"[{style}]{open_count} issue(s) remaining[/{style}]")


def print_complete(project_dir: Path, sessions: int | None = None, total_time: float | None = None) -> None:
    console.print("\n[bold green]── COMPLETE ──[/bold green]\n")
    console.print("[green]All issues done![/green]")
    if sessions and total_time:
        console.print(f"[dim]{sessions} sessions | {format_duration(total_time)}[/dim]")
    console.print()


def print_output(stdout: str, stderr: str) -> None:
    if stdout:
        console.print(Panel(Markdown(stdout.strip()), title="[cyan]Claude[/cyan]", border_style="cyan", padding=(1, 2)))
    if stderr:
        console.print(f"[red][stderr]: {stderr}[/red]")


def print_separator() -> None:
    console.print("\n" + "─" * 70 + "\n")


def print_timeout(timeout: int, duration: float | None = None) -> None:
    console.print(f"[red]Timeout ({timeout}s, ran {format_duration(duration or 0)})[/red]")


def print_error(e: Exception, duration: float | None = None, session_type: str | None = None) -> None:
    console.print(f"[red]Error: {e}[/red]")
    if duration or session_type:
        parts = [f"Duration: {format_duration(duration)}" if duration else "", session_type or ""]
        console.print(f"[dim]{' | '.join(p for p in parts if p)}[/dim]")


def print_warning(msg: str) -> None:
    console.print(f"[yellow]Warning: {msg}[/yellow]")


def print_max_sessions(n: int) -> None:
    console.print(f"\n[yellow]Reached max sessions ({n})[/yellow]")


def print_user_stopped() -> None:
    console.print("[yellow]Stopped. Run 'autonomous-claude --continue' to resume.[/yellow]")


class Spinner:
    def __init__(self, label: str = "Running..."):
        self._label = label

    def __enter__(self):
        self._progress = Progress(
            SpinnerColumn(), TextColumn(f"[cyan]{self._label}[/cyan]"), TimeElapsedColumn(),
            console=console, transient=True
        )
        self._progress.start()
        self._progress.add_task("", total=None)
        return self

    def __exit__(self, *_):
        self._progress.stop()


def wait_for_stop_signal(timeout: float = 10.0) -> bool:
    """Wait for keypress. Returns True if user wants to stop."""
    play_notification()

    if not sys.stdin.isatty():
        return False

    old = termios.tcgetattr(sys.stdin)
    try:
        tty.setraw(sys.stdin.fileno())
        start = time.time()
        remaining = timeout

        while remaining > 0:
            sys.stdout.write(f"\r\033[K  Press any key to stop, wait {remaining:.0f}s to continue... ")
            sys.stdout.flush()

            ready, _, _ = select.select([sys.stdin], [], [], min(1.0, remaining))
            if ready:
                sys.stdin.read(1)
                sys.stdout.write("\r\033[K")
                sys.stdout.flush()
                return True

            remaining = timeout - (time.time() - start)

        sys.stdout.write("\r\033[K")
        sys.stdout.flush()
        return False
    finally:
        termios.tcsetattr(sys.stdin, termios.TCSADRAIN, old)



import sys
from dataclasses import dataclass
from pathlib import Path

if sys.version_info >= (3, 11):
    import tomllib
else:
    try:
        import tomli as tomllib
    except ImportError:
        tomllib = None

CONFIG_FILE = Path.home() / ".config" / "autonomous-claude" / "config.toml"
AUTONOMOUS_CLAUDE_DIR = ".autonomous-claude"
SPEC_FILE = f"{AUTONOMOUS_CLAUDE_DIR}/spec.md"
LOGS_DIR = f"{AUTONOMOUS_CLAUDE_DIR}/logs"
SANDBOX_IMAGE = "ghcr.io/ferdousbhai/autonomous-claude"


@dataclass
class Config:
    timeout: int = 18000
    max_turns: int = 2000
    max_sessions: int = 100
    spec_timeout: int = 1800
    ralph_enabled: bool = True
    ralph_max_iterations: int = 10
    sandbox_enabled: bool = True
    sandbox_memory_limit: str = "8g"
    sandbox_cpu_limit: float = 4.0
    sandbox_image: str = SANDBOX_IMAGE
    sandbox_tag: str = ""
    notification_sound: str = "/usr/share/sounds/freedesktop/stereo/complete.oga"
    notification_dings: int = 5
    notification_interval: float = 0.3

    @classmethod
    def load(cls) -> "Config":
        config = cls()
        if not CONFIG_FILE.exists() or not tomllib:
            return config

        try:
            data = tomllib.loads(CONFIG_FILE.read_text())
        except Exception:
            return config

        mappings = {
            "session": ["timeout", "max_turns", "max_sessions", "spec_timeout"],
            "sandbox": {"enabled": "sandbox_enabled", "memory_limit": "sandbox_memory_limit",
                       "cpu_limit": "sandbox_cpu_limit", "image": "sandbox_image", "tag": "sandbox_tag"},
            "ralph": {"enabled": "ralph_enabled", "max_iterations": "ralph_max_iterations"},
            "notification": {"sound": "notification_sound", "dings": "notification_dings",
                           "interval": "notification_interval"},
        }

        for section, fields in mappings.items():
            if section not in data:
                continue
            if isinstance(fields, list):
                for f in fields:
                    if f in data[section]:
                        setattr(config, f, data[section][f])
            else:
                for file_key, attr in fields.items():
                    if file_key in data[section]:
                        setattr(config, attr, data[section][file_key])

        config.ralph_max_iterations = max(1, config.ralph_max_iterations)
        return config


_config: Config | None = None


def get_config() -> Config:
    global _config
    if _config is None:
        _config = Config.load()
    return _config
